unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  RDX, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    RenderDX1: TRenderDX;
    Memo1: TMemo;
    Label1: TLabel;
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Label1Click(Sender: TObject);
  private
    original_pal : hPalette;
    procedure AppIdle(Sender: TObject; var Done: Boolean);
  public
    // flags for direction of movement;
    Rotating : integer;
    Moving   : integer;
    Shearing : integer;


    // Amounts of movement
    Shear    : integer;   // looking up and down
    Velocity : integer;   // moving forward/backward
    Spin     : integer;   // angles turning
    MaxVelocity  : integer;

    function GetPalette : hPalette; override;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

function TForm1.GetPalette : hPalette;
begin
   result :=  hPalette(renderdx1.palette);
end;

procedure TForm1.AppIdle(Sender: TObject; var Done: Boolean);
var ctr : integer;
begin
    Done := False;
    // This really should be timed, but in the sake of readability,
    // we'll present it rather plainly.
    if moving <> 0 then begin
       // speed up
       inc(velocity,moving*4);
       if velocity >  maxvelocity then velocity :=  maxvelocity;
       if velocity < -maxvelocity then velocity := -maxvelocity;
    end else begin
       // slow down
       if velocity > 0 then dec(velocity,4);
       if velocity < 0 then inc(velocity,4);
    end;

    if rotating <> 0 then begin
       // speed up rotation
       inc(spin,rotating);
       if spin >  16 then spin :=  16;
       if spin < -16 then spin := -16;
    end else begin
       // slow down rotation
       if spin >  0 then dec(spin);
       if spin <  0 then inc(spin);
    end;

    if shearing <> 0 then begin
       // Look farther up and down
       inc(shear,shearing*5);
       if shear >  200 then shear :=  200;
       if shear < -200 then shear := -200;
    end else begin
       // Move view back toward center.
       if shear >  0 then dec(shear,5);
       if shear <  0 then inc(shear,5);
    end;

    // move viewpoint, spin, and shear the view.
    if velocity <> 0 then begin
       RenderDX1.MoveActor(0,velocity);
    end;
    if spin <> 0 then begin
       RenderDX1.SpinActor(0,Spin);
    end;
    RenderDX1.Shear := shear;
    RenderDX1.CheckActor(0);
    RenderDX1.Render(0,0);
    Application.ProcessMessages;
    RenderDX1.Repaint;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     case Key of
         VK_UP:    moving   :=  1;
         VK_DOWN:  moving   := -1;
         VK_LEFT:  rotating := -1;
         VK_RIGHT: rotating :=  1;
         VK_PRIOR: shearing := -1;
         VK_NEXT:  shearing :=  1;
     end;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     case Key of
         VK_UP:    moving   :=  0;
         VK_DOWN:  moving   :=  0;
         VK_LEFT:  rotating :=  0;
         VK_RIGHT: rotating :=  0;
         VK_PRIOR: shearing :=  0;
         VK_NEXT:  shearing :=  0;
     end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
    Application.OnIdle := AppIdle;
    MaxVelocity := 48;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    Application.OnIdle := nil;
end;

procedure TForm1.Label1Click(Sender: TObject);
begin
     if maxvelocity = 48 then begin
        label1.caption := 'Click here to speed up';
        maxvelocity := 24;
     end else begin
        label1.caption := 'Click here to slow down';
        maxvelocity := 48;
     end;
end;

end.
