unit Imagefrm;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls,
  FileCtrl, StdCtrls, ExtCtrls, Buttons, Spin, Dialogs, Grids, Outline,
  DirOutln;

type
  TEnhancedImageForm = class(TForm)
    DriveComboBox1: TDriveComboBox;
    PathLabel: TLabel;
    FileEdit: TEdit;
    UpDownGroup: TGroupBox;
    SpeedButton1: TSpeedButton;
    BitBtn1: TBitBtn;
    DisabledGrp: TGroupBox;
    SpeedButton2: TSpeedButton;
    BitBtn2: TBitBtn;
    Panel1: TPanel;
    Image1: TImage;
    FileListBox1: TFileListBox;
    Label1: TLabel;
    SpinEdit1: TSpinEdit;
    Label2: TLabel;
    ViewBtn: TBitBtn;
    Bevel1: TBevel;
    Bevel2: TBevel;
    FilterComboBox1: TFilterComboBox;
    CheckBox1: TCheckBox;
    StretchCheck: TCheckBox;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    cmbQuickDirs: TComboBox;
    SpeedButton3: TSpeedButton;
    Label3: TLabel;
    BitBtn5: TBitBtn;
    SaveDialog1: TSaveDialog;
    BitBtn6: TBitBtn;
    WaitLabel: TLabel;
    DirectoryOutline1: TDirectoryOutline;
    SpeedButton4: TSpeedButton;
    procedure FileListBox1Click(Sender: TObject);
    procedure SpinEdit1Change(Sender: TObject);
    procedure ViewBtnClick(Sender: TObject);
    procedure ViewAsGlyph(const FileExt: string);
    procedure CheckBox1Click(Sender: TObject);
    procedure StretchCheckClick(Sender: TObject);
    procedure FileEditKeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cmbQuickDirsChange(Sender: TObject);
    procedure BitBtn5Click(Sender: TObject);
    procedure BitBtn6Click(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure BitBtn3Click(Sender: TObject);
    procedure DirectoryOutline1Change(Sender: TObject);
    procedure DriveComboBox1Change(Sender: TObject);
    procedure FileListBox1Change(Sender: TObject);
    procedure FileListBox1DblClick(Sender: TObject);
    procedure SpeedButton4Click(Sender: TObject);
  private
    { Private declarations }
      DefaultDir : string;
      strlDirectories : TStringList;
    procedure LoadQuickDirs;
  end;

var
  EnhancedImageForm: TEnhancedImageForm;

implementation

uses SysUtils, ViewFrm, IniFiles, Quickdir;

{$R *.DFM}

const
  DefaultAppName = 'Default application directory';

function IsDirectory (const Name: string) : Boolean;
begin
  Result := (Name[1] = '[') and (Name[Length(Name)] = ']');
end;

procedure TEnhancedImageForm.LoadQuickDirs;
var
  IniFile : TIniFile;
  QuickDirCount, i, LastDirIndex : Integer;
begin
  IniFile := TIniFile.Create('Pictedit.ini');
  with IniFile do
  try
    cmbQuickDirs.Clear;
    cmbQuickDirs.Items.Add(DefaultAppName);
    strlDirectories.Clear;
    strlDirectories.Add(DefaultDir);
    QuickDirCount := ReadInteger('General', 'QuickDirCount', 0);
    for i := 1 to QuickDirCount do
    begin
      cmbQuickDirs.Items.Add(ReadString('Long names', 'Directory' + IntToStr(i),
        'No name available'));
      strlDirectories.Add(ReadString('Path', 'Directory' + IntToStr(i),
        DefaultDir));
    end; { for }
    LastDirIndex := ReadInteger('General', 'LastDirIndex', 0);
    cmbQuickDirs.ItemIndex := LastDirIndex;
    if LastDirIndex > -1 then
      cmbQuickDirsChange(Self);
    CheckBox1.Checked := ReadBool('General', 'ViewAsGlyph', True);
    StretchCheck.Checked := ReadBool('General', 'Stretch', False);
  finally
    IniFile.Free;
  end; { finally }
end;

procedure TEnhancedImageForm.FileListBox1Click(Sender: TObject);
var
  FileExt: string[4];
begin
  FileExt := UpperCase(ExtractFileExt(FileListBox1.Filename));
  if (FileExt = '.BMP') or (FileExt = '.ICO') or (FileExt = '.WMF') then
  begin
    Image1.Picture.LoadFromFile(FileListBox1.Filename);
    Label1.Caption := ExtractFilename(FileListBox1.Filename);
    if (FileExt = '.BMP') then
      begin
        Label3.Caption := Format('(%d x %d)', [Image1.Picture.Width,
          Image1.Picture.Height]);
        ViewForm.Image1.Picture := Image1.Picture;
        ViewAsGlyph(FileExt);
      end { if }
    else
      Label3.Caption := '(n/a)';
    if FileExt = '.ICO' then Icon := Image1.Picture.Icon;
    if FileExt = '.WMF' then
      ViewForm.Image1.Picture.Metafile := Image1.Picture.Metafile;
  end;
end;

procedure TEnhancedImageForm.CheckBox1Click(Sender: TObject);
begin
  if CheckBox1.Checked then
    ViewAsGlyph(UpperCase(ExtractFileExt(FileListBox1.Filename)))
  else
    begin
      DeleteObject(BitBtn1.Glyph.ReleaseHandle);
      BitBtn1.Glyph := nil;
      DeleteObject(BitBtn2.Glyph.ReleaseHandle);
      BitBtn2.Glyph := nil;
      DeleteObject(SpeedButton1.Glyph.ReleaseHandle);
      SpeedButton1.Glyph := nil;
      DeleteObject(SpeedButton2.Glyph.ReleaseHandle);
      SpeedButton2.Glyph := nil;
    end; { else }
end;

procedure TEnhancedImageForm.ViewAsGlyph(const FileExt: string);
begin
  if CheckBox1.Checked and (FileExt = '.BMP') then
  begin
    SpeedButton1.Glyph := Image1.Picture.Bitmap;
    SpeedButton2.Glyph := Image1.Picture.Bitmap;
    SpinEdit1.Value := SpeedButton1.NumGlyphs;
    BitBtn1.Glyph := Image1.Picture.Bitmap;
    BitBtn2.Glyph := Image1.Picture.Bitmap;
  end;
end;

procedure TEnhancedImageForm.ViewBtnClick(Sender: TObject);
begin
  ViewForm.HorzScrollBar.Range := Image1.Picture.Width;
  ViewForm.VertScrollBar.Range := Image1.Picture.Height;
  ViewForm.Caption := Label1.Caption;
  ViewForm.Show;
end;

procedure TEnhancedImageForm.SpinEdit1Change(Sender: TObject);
begin
  SpeedButton1.NumGlyphs := SpinEdit1.Value;
  SpeedButton2.NumGlyphs := SpinEdit1.Value;
  BitBtn1.NumGlyphs := SpinEdit1.Value;
  BitBtn2.NumGlyphs := SpinEdit1.Value;
end;

procedure TEnhancedImageForm.StretchCheckClick(Sender: TObject);
begin
  Image1.Stretch := StretchCheck.Checked;
end;

procedure TEnhancedImageForm.FileEditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
  begin
    FileListBox1.ApplyFilePath(FileEdit.Text);
    Key := #0;
  end;
end;

procedure TEnhancedImageForm.FormShow(Sender: TObject);
begin
  GetDir(0, DefaultDir);
  LoadQuickDirs;
end;

procedure TEnhancedImageForm.FormCreate(Sender: TObject);
begin
  strlDirectories := TStringList.Create;
end;

procedure TEnhancedImageForm.FormDestroy(Sender: TObject);
begin
  ChDir(DefaultDir);
  strlDirectories.Free;
end;

procedure TEnhancedImageForm.cmbQuickDirsChange(Sender: TObject);
begin
  PathLabel.Visible := False;
  WaitLabel.Visible := True;
  Application.ProcessMessages;
  try
    if strlDirectories[cmbQuickDirs.ItemIndex] = '' then
      raise Exception.Create('Invalid directory');
    DirectoryOutline1.Directory := strlDirectories[cmbQuickDirs.ItemIndex];
  except
    MessageDlg('The entry "' + cmbQuickDirs.Items[cmbQuickDirs.ItemIndex] +
      '" does not point to a valid directory.   Please verify.', mtError,
      [mbOk], 0);
  end; { except }
  Application.ProcessMessages;
  WaitLabel.Visible := False;
  PathLabel.Visible := True;
end;

procedure TEnhancedImageForm.BitBtn5Click(Sender: TObject);
begin
  if Label1.Caption <> '' then
    SaveDialog1.Filename := Label1.Caption;
  if SaveDialog1.Execute then
    Image1.Picture.SaveToFile(SaveDialog1.Filename)
  else
    begin
      ModalResult := mrNone;
      cmbQuickDirs.SetFocus;
    end; { else }
end;

procedure TEnhancedImageForm.BitBtn6Click(Sender: TObject);
var
  DirDescription : string;
  IniFile : TIniFile;
begin
  DirDescription := '';
  if InputQuery('Adding new Quick Directory', 'Enter name for Quick Directory:',
    DirDescription) then
  begin
    cmbQuickDirs.Items.Add(DirDescription);
    strlDirectories.Add(DirectoryOutline1.Directory);
    cmbQuickDirs.ItemIndex := Pred(cmbQuickDirs.Items.Count);
    IniFile := TIniFile.Create('Pictedit.ini');
    with IniFile do
    try
      WriteInteger('General', 'QuickDirCount', Pred(cmbQuickDirs.Items.Count));
      WriteString('Long names', 'Directory' +
        IntToStr(Pred(cmbQuickDirs.Items.Count)), DirDescription);
      WriteString('Path', 'Directory' +
        IntToStr(Pred(cmbQuickDirs.Items.Count)), DirectoryOutline1.Directory);
    finally
      IniFile.Free;
    end; { finally }
  end; { if }
end;

procedure TEnhancedImageForm.SpeedButton3Click(Sender: TObject);
begin
  frmQuickDirs := TfrmQuickDirs.Create(Self);
  with frmQuickDirs do
  try
    lstbDirectories.Items.AddStrings(cmbQuickDirs.Items);
    lstbDirectories.Items.Delete(0);
    strlDirectories.AddStrings(Self.strlDirectories);
    strlDirectories.Delete(0);
    if (ShowModal = mrOk) then
    with Self do
    begin
      cmbQuickDirs.Items.Clear;
      cmbQuickDirs.Items.AddStrings(frmQuickDirs.lstbDirectories.Items);
      cmbQuickDirs.Items.Insert(0, DefaultAppName);
      cmbQuickDirs.ItemIndex := 0;
      strlDirectories.Clear;
      strlDirectories.AddStrings(frmQuickDirs.strlDirectories);
      strlDirectories.Insert(0, DefaultDir);
      DirectoryOutline1.Directory := DefaultDir;
    end; { with }
  finally
    frmQuickDirs.Free;
  end; { finally }
end;

procedure TEnhancedImageForm.BitBtn3Click(Sender: TObject);
var
  IniFile : TIniFile;
begin
  IniFile := TIniFile.Create('Pictedit.ini');
  with IniFile do
  try
    WriteInteger('General', 'LastDirIndex', cmbQuickDirs.ItemIndex);
    WriteBool('General', 'ViewAsGlyph', CheckBox1.Checked);
    WriteBool('General', 'Stretch', StretchCheck.Checked);
  finally
    IniFile.Free;
  end; { finally }
end;

procedure TEnhancedImageForm.DirectoryOutline1Change(Sender: TObject);
begin
  PathLabel.Visible := False;
  WaitLabel.Visible := True;
  Application.ProcessMessages;
  try
    FileListBox1.Directory := DirectoryOutline1.Directory;
    PathLabel.Caption := DirectoryOutline1.Directory;
    DriveComboBox1.Drive := DirectoryOutline1.Drive;
  finally
    PathLabel.Visible := True;
    WaitLabel.Visible := False;
  end; { finally }
end;

procedure TEnhancedImageForm.DriveComboBox1Change(Sender: TObject);
begin
  DirectoryOutline1.Drive := DriveComboBox1.Drive;
end;

procedure TEnhancedImageForm.FileListBox1Change(Sender: TObject);
begin
  if FileListBox1.ItemIndex <> -1 then
    FileEdit.Text := ExtractFileName(FileListBox1.FileName)
  else
    if Copy(FileListBox1.Mask, Length(FileListBox1.Mask) - 3, 4) = ';*. ' then
      FileEdit.Text := Copy(FileListBox1.Mask, 1, Length(FileListBox1.Mask) - 4)
    else
      FileEdit.Text := FileListBox1.Mask;
end;

procedure TEnhancedImageForm.FileListBox1DblClick(Sender: TObject);
begin
  with FileEdit do
  begin
    if IsDirectory(Text) then
      DirectoryOutline1.Directory := Copy(Text, 2, Length(Text) - 2);
  end; { with }
end;

procedure TEnhancedImageForm.SpeedButton4Click(Sender: TObject);
begin
  Image1.Picture := nil;
  Icon := nil;
end;

end.
